/*
    AQTRONIX C++ Library
    Copyright 2004-2007 Parcifal Aertssen

    This file is part of AQTRONIX C++ Library.

    AQTRONIX C++ Library is free software; you can redistribute it
	and/or modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.

    AQTRONIX C++ Library is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied warranty
    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AQTRONIX C++ Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
*/
// ClientSocket.cpp: implementation of the CClientSocket class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ClientSocket.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/* CHANGELOG
 *
 *  2007.02.11 Fixed problem in Receive() not return full response
 *	2004 Class created
 */

#define BUFLENGTH 1024

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CClientSocket::CClientSocket(): ArchiveIn(NULL),ArchiveOut(NULL),SocketFile(NULL)
{

}

CClientSocket::~CClientSocket()
{
	DestroyArchives();
}

BOOL CClientSocket::CreateAsync(UINT nSocketPort,int nSocketType,long lEvent)
{
	BOOL ret = CAsyncSocket::Create(nSocketPort,nSocketType,lEvent);
	CreateArchives();
	return ret;
}

BOOL CClientSocket::CreateSync(UINT nSocketPort,int nSocketType,LPCTSTR lpszSocketAddress)
{
	BOOL ret = CSocket::Create(nSocketPort,nSocketType,lpszSocketAddress);
	CreateArchives();
	return ret;
}

CString CClientSocket::Receive()
{
	/*
	if(ArchiveIn==NULL)
		CreateArchives();

	if(ArchiveIn!=NULL){
		if(ArchiveIn->IsBufferEmpty())
			rcv += "empty";
		while(!ArchiveIn->IsBufferEmpty()){
			rcv += ">>";
			*ArchiveIn >> rcv;
		}
	}else{
		rcv = "ArchiveIn==NULL";
	}//*/

	char buf[BUFLENGTH+1];
	int length;

	//read first chunk
	length = CSocket::Receive(buf,BUFLENGTH);
	buf[length] = '\0';	//null terminate
	CString rcv = buf;
	//read next chunks
	while(length>0){
		length = CSocket::Receive(buf,BUFLENGTH);
		buf[length] = '\0';	//null terminate
		rcv += buf;
	}

	//Send("input>");
	
	return rcv;
}

bool CClientSocket::Send(const char *Packet)
{
	try{

		/*
		if(ArchiveOut==NULL)
			CreateArchives();

		if(Packet!=NULL && ArchiveOut!=NULL){
			*ArchiveOut << *Packet;
			return true;
		}else{
			return false;
		}
		//*/

		
		if(Packet==NULL){
			return false;
		}else{
			size_t length(strlen(Packet));
			size_t bytes;
			bytes = CAsyncSocket::Send(Packet,length);
			if(bytes==0){
				return false;
			}else{
				size_t oldlength;
				while(bytes<length){
					oldlength=length;
					length -= bytes;
					bytes = CAsyncSocket::Send((&Packet+bytes),length);
					if(bytes==0)
						return true;
				}
				return true;
			}
		}//*/
	}catch(...){
		return false;
	}
}

void CClientSocket::OnReceive(int nErrorCode)
{
	if(!nErrorCode)
		Receive();
	//CSocket::OnReceive(nErrorCode);

	/*
	if (ProcessPendingRead()) {
		ProcessReq() ; // process the request
		m_bDone = TRUE ;
		if (GetMyProps()->m_bLogEnable)
			m_pDoc->WriteLog(m_LogRec);
		m_pDoc->KillSocket(this);
	}
	//*/
}

BOOL CClientSocket::OnMessagePending(){
	MSG msg;
	if(::PeekMessage(&msg, NULL, WM_TIMER, WM_TIMER, PM_NOREMOVE)){
		if (msg.wParam == (UINT) TimerID){
			//Remove the message and call CancelBlockingCall
			::PeekMessage(&msg, NULL, WM_TIMER, WM_TIMER, PM_REMOVE);
			CancelBlockingCall();
			return FALSE;  // No need for idle time processing
		}
	}
	return CSocket::OnMessagePending();
}

BOOL CClientSocket::SetTimeOut(UINT uTimeOut){
	//set timer
	TimerID = SetTimer(NULL,0,uTimeOut,NULL);
	return TimerID>0;
}

BOOL CClientSocket::KillTimeOut(){
	//kill timer
	return KillTimer(NULL,TimerID);
}

BOOL CClientSocket::Connect(LPCTSTR Host, unsigned int Port)
{
	if(CSocket::Connect(Host,Port)){
		return TRUE;
	}else{

		if(GetLastError()==WSAENOTSOCK)
			AfxMessageBox("WSAENOTSOCK");
	
		return FALSE;
	}
}

void CClientSocket::CreateArchives(bool Sending, bool Receiving)
{
	//socketfile
	SocketFile = new CSocketFile(this);
	//archives
	if(Sending)
		ArchiveOut = new CArchive(SocketFile,CArchive::store);
	if(Receiving)
		ArchiveIn = new CArchive(SocketFile,CArchive::load);
}

void CClientSocket::DestroyArchives()
{
	//delete archives
	if(ArchiveIn!=NULL)
		delete ArchiveIn;
	if(ArchiveOut!=NULL)
		delete ArchiveOut;

	//delete socketfile
	if(SocketFile!=NULL)
		delete SocketFile;
}
