#include "stdafx.h"
#include "LogAnalysisSettings.h"

#define SEPARATOR1		" ; "
#define SEPARATOR2		"\t"
#define SEPARATOR3		"\";\"" //";"
#define SEPARATOR4		"\",\""	//","
#define SEPARATOR5		"\" \"" //" "
#define SEPARATOR6		";"
#define SEPARATOR7		","
#define SEPARATOR8		" "

CLogAnalysisSettings::CLogAnalysisSettings(void)
{
	LoadDefaults();
}

CLogAnalysisSettings::~CLogAnalysisSettings(void)
{
}

void CLogAnalysisSettings::LoadDefaults()
{
	General.EOL = "\r\n";
	General.Separator = "";
	General.Trim = false;
	General.FirstLineHeader = false;
	General.SortedColumn = -1;
	Filter.MaxEntries = 500000; //INT_MAX;
	Filter.Include.RemoveAll();
	Filter.Exclude.RemoveAll();
	Filter.Regex.RemoveAll();
}

bool CLogAnalysisSettings::WriteToFileINI(LPCTSTR fn)
{
	//Response handling
	//WRITE_INI_STRING(General,EOL)
	WRITE_INI_STRING(General,Separator)
	WRITE_INI_BOOL(General,Trim)
	WRITE_INI_BOOL(General,FirstLineHeader)
	WRITE_INI_INT(General,SortedColumn)
	WRITE_INI_INT(Filter,MaxEntries)
	WRITE_INI_CSTRINGLIST(Filter,Include)
	WRITE_INI_CSTRINGLIST(Filter,Exclude)
	//WRITE_INI_CSTRINGLIST(Filter,Regex) //TODO: FILTERS - regex

	return true;
}

bool CLogAnalysisSettings::ReadFromFileINI(LPCTSTR fn)
{
	const int sz = MAX_INI_FILE_SIZE;
	char buf[sz];
	DWORD l(0);

	//READ_INI_STRING(General,EOL)
	READ_INI_STRING(General,Separator)
	READ_INI_BOOL(General,Trim)
	READ_INI_BOOL(General,FirstLineHeader)
	READ_INI_INT(General,SortedColumn)
	READ_INI_INT(Filter,MaxEntries)
	READ_INI_CSTRINGLIST(Filter,Include,)
	READ_INI_CSTRINGLIST(Filter,Exclude,)
	READ_INI_CSTRINGLIST(Filter,Regex,)

	return true;
}

bool CLogAnalysisSettings::ParseXML(CString &key, CString &val)
{
	//if not a valid key
	if (key=="")
		return false;

	CString name = XMLDecode(CXML::TagName(key));
	
	//Response Handling
	//	 PARSE_XML_STRING(General,EOL,"EOL")
	/*else*/ PARSE_XML_STRING(General,Separator,"Separator")
	else PARSE_XML_BOOL(General,Trim,"Trim")
	else PARSE_XML_BOOL(General,FirstLineHeader,"FirstLineHeader")
	else PARSE_XML_INT(General,SortedColumn,"SortedColumn",)
	else PARSE_XML_INT(Filter,MaxEntries,"MaxEntries",)
	else PARSE_XML_CSTRINGLIST(Filter,Include,"Filter_Include")
	else PARSE_XML_CSTRINGLIST(Filter,Exclude,"Filter_Exclude")
	else PARSE_XML_CSTRINGLIST(Filter,Regex,"Filter_Regex")

	else{
		return false;
	}
	return true;
}

CString CLogAnalysisSettings::ToXML()
{
	CString xml("");
	CLogAnalysisSettings d;

	//WRITE_XML_STRING(General,EOL,"EOL","The End-Of-Line character or string")
	WRITE_XML_STRING(General,Separator,"Separator","The separator for the fields")
	WRITE_XML_BOOL(General,Trim,"Trim","Trim whitespaces and quotes")
	WRITE_XML_BOOL(General,FirstLineHeader,"FirstLineHeader","First line is the header")
	WRITE_XML_INT(General,SortedColumn,"SortedColumn","The column index to be sorted ascending")
	WRITE_XML_INT(Filter,MaxEntries,"MaxEntries","The maximum number of entries shown in the list")
	WRITE_XML_CSTRINGLIST(Filter,Include,"Filter_Include","Only see entries with these strings")
	WRITE_XML_CSTRINGLIST(Filter,Exclude,"Filter_Exclude","Ignore entries with these strings")
	WRITE_XML_CSTRINGLIST(Filter,Regex,"Filter_Regex","Only see entries matching these regex patterns (syntax CAtlRegExp)")

	return xml;
}

bool CLogAnalysisSettings::WriteToFileXML(LPCTSTR fn)
{
	return WriteXMLEntityToFile(fn,"LogAnalysis",ToXML());
}

bool CLogAnalysisSettings::IsAllowedByFilters(CString& Line)
{
	//TODO: FILTERS - case-sensitive/insensitive "Match case" in UI
	if(Filter.Include.GetCount()>0){
		CStringList Matches;
		if(Filter.Include.GetCount()!=CStringHelper::CountKeywords(Line,Filter.Include,Matches)){
			return false;
		}
	}
	if(Filter.Exclude.GetCount()>0){
		if(CStringHelper::IsListItemInStringCompareNoCase(Line,Filter.Exclude)){
			return false;
		}
	}

	//TODO: FILTERS - regex
	//if(Filter.Regex.GetCount()>0){
	//	POSITION pos = Filter.Regex.GetHeadPosition();
	//	CString pattern;
	//	while(pos!=NULL){
	//		pattern = Filter.Regex.GetNext(pos);
	//		CAtlRegExp<>* regex = regex_cache.Lookup(pattern);
	//		if(regex!=NULL){ //NULL = regex syntax error
	//			CAtlREMatchContext<> mc;
	//			if(!regex->Match(Line,&mc)){
	//				return false;
	//			}
	//		}
	//	}
	//}

	return true;
}

CString CLogAnalysisSettings::DetectSeparator(CString& Line)
{
	if(Line.Find(SEPARATOR1,0)!=-1)
		return SEPARATOR1;

	if(Line.Find(SEPARATOR2,0)!=-1)
		return SEPARATOR2;

	if(Line.Find(SEPARATOR3,0)!=-1)
		return SEPARATOR3;

	if(Line.Find(SEPARATOR4,0)!=-1)
		return SEPARATOR4;

	if(Line.Find(SEPARATOR5,0)!=-1)
		return SEPARATOR5;

	if(Line.Find(SEPARATOR6,0)!=-1)
		return SEPARATOR6;

	if(Line.Find(SEPARATOR7,0)!=-1)
		return SEPARATOR7;

	if(Line.Find(SEPARATOR8,0)!=-1)
		return SEPARATOR8;

	return ""; //if no header separator found, use complete line as header
}

CString CLogAnalysisSettings::DetectEOL(CString Text)
{
	if(Text.Find("\r\n",0)!=-1) //DOS
		return "\r\n";

	if(Text.Find("\n",0)!=-1) //Unix
		return "\n";

	if(Text.Find("\r",0)!=-1) //Old Mac
		return "\r";

	return "\r\n"; //if no EOL separator found, use CRLF
}